using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

/// <summary>
/// A simple example to show how a timer can be used to animated a change 
/// in an object property such as color.  Even though no threads are used,
/// the animation still responds to real-time, meaning that on a fast computer,
/// there are more inbetween frames.
/// 
/// Ben Bederson
/// University of Maryland
/// May 2005
/// </summary>
namespace WindowsApplication1 {
	public class Form1 : System.Windows.Forms.Form {
		private System.ComponentModel.Container components = null;

		private Color fromColor = Color.Black;
		private Color toColor = Color.Red;
		private SolidBrush brush = new SolidBrush(Color.Black);
		private Font font = new Font("Arial", 20);

		private DateTime start;
		private int duration = 1000;
		private int numRenders = 0;

		public Form1() {
			SetStyle(ControlStyles.DoubleBuffer, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.UserPaint, true);

			InitializeComponent();
			Click += new EventHandler(Form1_Click);

			StartAnimation();
		}

		void StartAnimation() {
			numRenders = 0;
			start = DateTime.Now;
			Invalidate();
		}

		protected override void OnPaint(PaintEventArgs e) {
			base.OnPaint (e);

			bool done = UpdateBrush();
			e.Graphics.FillRectangle(brush, ClientRectangle);
			if (!done) {
				Invalidate();
			}

			// Debugging code to display frames per second in display
			DateTime now = DateTime.Now;
			TimeSpan elapsedSpan = now.Subtract(start);
			float elapsed = (elapsedSpan.Seconds * 1000.0f + elapsedSpan.Milliseconds) / 1000.0f;
			numRenders++;
			int fps = (int)(numRenders / elapsed);
			e.Graphics.DrawString(fps.ToString(), font, Brushes.Yellow, 50, 50);
			// Uncomment the next line to force rendering to be slow
			// System.Threading.Thread.Sleep(200);
		}

		private bool UpdateBrush() {
			DateTime now = DateTime.Now;
			TimeSpan elapsed = now.Subtract(start);
			float t = (elapsed.Seconds * 1000.0f + elapsed.Milliseconds) / duration;
			if (t > 1) {
				t = 1;
			}
			int r = (int)LERP(t, fromColor.R, toColor.R);
			int g = (int)LERP(t, fromColor.G, toColor.G);
			int b = (int)LERP(t, fromColor.B, toColor.B);
			brush.Color = Color.FromArgb(r, g, b);
			
			return (t >= 1);
		}

		private float LERP(float t, float p1, float p2) {
			float p = p1 + t * (p2 - p1);
			return p;
		}

		private void Form1_Click(object sender, EventArgs e) {
			StartAnimation();
		}

		#region Dispose
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			this.Size = new System.Drawing.Size(300,300);
			this.Text = "Form1";
		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new Form1());
		}
	}
}